You can find below the conversation history between the user and the assistant.

<conversation-history>
{% for message in conversation_history %}
{{message.role}}: {{message.content}}
{% endfor %}
</conversation-history>

You can find below some additional context that might be interesting to generate follow-up questions from:

{% if additional_context %}
<additional-context>
{{additional_context}}
</additional-context>
{% endif %}

{% if language %}
Generate the follow-up questions in {{language}} language.
Remember that you should not repeat questions that you have seen in the conversation history.
{% endif %}